@@ -231,9 +231,9 @@ def task_create(session, values):
 
     if tags:
         session.bulk_save_objects(
-            models.Tag(uuid=task["uuid"], tag=t,
-                       type=consts.TagType.TASK)
-            for t in set(tags))
+            [models.Tag(uuid=task["uuid"], tag=t,
+                        type=consts.TagType.TASK)
+             for t in set(tags)])
     task["tags"] = tags
     return task
 
@@ -264,8 +264,8 @@ def task_update(session, uuid, values):
 
         if new_tags:
             session.bulk_save_objects(
-                models.Tag(uuid=uuid, tag=t, type=consts.TagType.TASK)
-                for t in set(new_tags))
+                [models.Tag(uuid=uuid, tag=t, type=consts.TagType.TASK)
+                 for t in set(new_tags)])
         task["tags"] = tags
     else:
         task["tags"] = []
@@ -718,9 +718,9 @@ def verification_create(session, verifier_id, env, tags=None, run_args=None):
 
     if tags:
         session.bulk_save_objects(
-            models.Tag(uuid=verification.uuid, tag=t,
-                       type=consts.TagType.VERIFICATION)
-            for t in set(tags)
+            [models.Tag(uuid=verification.uuid, tag=t,
+                        type=consts.TagType.VERIFICATION)
+             for t in set(tags)]
         )
     return verification
 
