@@ -16,11 +16,11 @@ import abc
 import bisect
 import collections
 import math
-
 import six
 
 from rally.common.plugin import plugin
 from rally.common import streaming_algorithms as streaming
+from rally.task import atomic
 from rally.task.processing import utils
 
 
@@ -38,17 +38,15 @@ class Chart(plugin.Plugin):
     def widget(self):
         """Widget name to display this chart by JavaScript."""
 
-    def __init__(self, workload_info, zipped_size=1000):
+    def __init__(self, workload, zipped_size=1000):
         """Setup initial values.
 
-        :param workload_info: dict, generalized info about iterations.
-                               The most important value is `iterations_count'
-                               that should have int value of total data size
+        :param workload: dict, detailed info about the Workload
         :param zipped_size: int maximum number of points on scale
         """
         self._data = collections.OrderedDict()  # Container for results
-        self._workload_info = workload_info
-        self.base_size = workload_info.get("iterations_count", 0)
+        self._workload = workload
+        self.base_size = self._workload["total_iteration_count"]
         self.zipped_size = zipped_size
 
     def add_iteration(self, iteration):
@@ -70,20 +68,30 @@ class Chart(plugin.Plugin):
         return [(name, points.get_zipped_graph())
                 for name, points in self._data.items()]
 
-    def _fix_atomic_actions(self, iteration):
+    @classmethod
+    def render_complete_data(cls, data):
+        """render processed complete data for drawing."""
+        return data
+
+    def _fix_atomic_actions(self, atomic_actions):
         """Set `0' for missed atomic actions.
 
         Since some atomic actions can absent in some iterations
         due to failures, this method must be used in all cases
         related to atomic actions processing.
         """
-        for name in self._workload_info["atomic"]:
-            iteration["atomic_actions"].setdefault(name, 0)
-        return iteration
+        return list(
+            (name, atomic_actions.get(name, {}).get("duration", 0))
+            for name in self._get_atomic_names()
+        )
+
+    def _get_atomic_names(self):
+        duration_stats = self._workload["statistics"]["durations"]
+        return [a["display_name"] for a in duration_stats["atomics"]]
 
-    @abc.abstractmethod
     def _map_iteration_values(self, iteration):
         """Get values for processing, from given iteration."""
+        return iteration
 
 
 class MainStackedAreaChart(Chart):
@@ -93,14 +101,14 @@ class MainStackedAreaChart(Chart):
     def _map_iteration_values(self, iteration):
         if iteration["error"]:
             result = [("duration", 0), ("idle_duration", 0)]
-            if self._workload_info["iterations_failed"]:
+            if self._workload["failed_iteration_count"]:
                 result.append(
                     ("failed_duration",
                      iteration["duration"] + iteration["idle_duration"]))
         else:
             result = [("duration", iteration["duration"]),
                       ("idle_duration", iteration["idle_duration"])]
-            if self._workload_info["iterations_failed"]:
+            if self._workload["failed_iteration_count"]:
                 result.append(("failed_duration", 0))
         return result
 
@@ -110,9 +118,10 @@ class AtomicStackedAreaChart(Chart):
     widget = "StackedArea"
 
     def _map_iteration_values(self, iteration):
-        iteration = self._fix_atomic_actions(iteration)
-        atomics = list(iteration["atomic_actions"].items())
-        if self._workload_info["iterations_failed"]:
+        atomic_actions = atomic.merge_atomic_actions(
+            iteration["atomic_actions"])
+        atomics = self._fix_atomic_actions(atomic_actions)
+        if self._workload["failed_iteration_count"]:
             if iteration["error"]:
                 failed_duration = (
                     iteration["duration"] + iteration["idle_duration"]
@@ -141,8 +150,9 @@ class AvgChart(Chart):
 class AtomicAvgChart(AvgChart):
 
     def _map_iteration_values(self, iteration):
-        iteration = self._fix_atomic_actions(iteration)
-        return list(iteration["atomic_actions"].items())
+        atomic_actions = atomic.merge_atomic_actions(
+            iteration["atomic_actions"])
+        return self._fix_atomic_actions(atomic_actions)
 
 
 class LoadProfileChart(Chart):
@@ -150,20 +160,19 @@ class LoadProfileChart(Chart):
 
     widget = "StackedArea"
 
-    def __init__(self, workload_info, name="parallel iterations",
+    def __init__(self, workload, name="parallel iterations",
                  scale=100):
         """Setup chart with graph name and scale.
 
-        :workload_info:  dict, generalized info about iterations
+        :param workload:  dict, detailed information about Workload
         :param name: str name for X axis
         :param scale: int number of X points
         """
-        super(LoadProfileChart, self).__init__(workload_info)
+        super(LoadProfileChart, self).__init__(workload)
         self._name = name
         # NOTE(boris-42): Add 2 points at the end of graph so at the end of
         #                 graph there will be point with 0 running iterations.
-        self._duration = workload_info["load_duration"] * (1 + 2.0 / scale)
-        self._tstamp_start = workload_info["tstamp_start"]
+        self._duration = self._workload["load_duration"] * (1 + 2.0 / scale)
 
         self.step = self._duration / float(scale)
         self._time_axis = [self.step * x
@@ -171,9 +180,16 @@ class LoadProfileChart(Chart):
                            if (self.step * x) < self._duration]
         self._time_axis.append(self._duration)
         self._running = [0] * len(self._time_axis)
+        # NOTE(andreykurilin): There is a "start_time" field in workload
+        #   object, but due to transformations in database layer, the
+        #   microseconds can be not accurate enough.
+        if self._workload["data"]:
+            self._tstamp_start = self._workload["data"][0]["timestamp"]
+        else:
+            self._tstamp_start = self._workload["start_time"]
 
     def _map_iteration_values(self, iteration):
-        return (iteration["timestamp"], iteration["duration"])
+        return iteration["timestamp"], iteration["duration"]
 
     def add_iteration(self, iteration):
         timestamp, duration = self._map_iteration_values(iteration)
@@ -257,8 +273,8 @@ class MainHistogramChart(HistogramChart):
 
     def __init__(self, workload_info):
         super(MainHistogramChart, self).__init__(workload_info)
-        views = self._init_views(self._workload_info["min_duration"],
-                                 self._workload_info["max_duration"])
+        views = self._init_views(self._workload["min_duration"],
+                                 self._workload["max_duration"])
         self._data["task"] = {"views": views, "disabled": None}
 
     def _map_iteration_values(self, iteration):
@@ -269,16 +285,17 @@ class AtomicHistogramChart(HistogramChart):
 
     def __init__(self, workload_info):
         super(AtomicHistogramChart, self).__init__(workload_info)
-        for i, atomic in enumerate(self._workload_info["atomic"].items()):
-            name, value = atomic
-            self._data[name] = {
-                "views": self._init_views(value["min_duration"],
-                                          value["max_duration"]),
+        for i, aa in enumerate(
+                self._workload["statistics"]["durations"]["atomics"]):
+            self._data[aa["display_name"]] = {
+                "views": self._init_views(aa["data"]["min"],
+                                          aa["data"]["max"]),
                 "disabled": i}
 
     def _map_iteration_values(self, iteration):
-        iteration = self._fix_atomic_actions(iteration)
-        return list(iteration["atomic_actions"].items())
+        atomic_actions = atomic.merge_atomic_actions(
+            iteration["atomic_actions"])
+        return self._fix_atomic_actions(atomic_actions)
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -299,6 +316,7 @@ class Table(Chart):
     """
 
     widget = "Table"
+    _styles = {}
 
     @abc.abstractproperty
     def columns(self):
@@ -330,6 +348,14 @@ class Table(Chart):
                 return ins.result() is not None
         return True
 
+    def _process_row(self, name, values):
+        row = [name]
+        has_result = self._row_has_results(values)
+        for ins, fn in values:
+            fn = fn or self._round
+            row.append(fn(ins, has_result))
+        return row
+
     def get_rows(self):
         """Collect rows values finally, after all data is processed.
 
@@ -337,16 +363,22 @@ class Table(Chart):
         """
         rows = []
         for name, values in self._data.items():
-            row = [name]
-            has_result = self._row_has_results(values)
-            for ins, fn in values:
-                fn = fn or self._round
-                row.append(fn(ins, has_result))
-            rows.append(row)
+            rows.append(self._process_row(name, values))
         return rows
 
     def render(self):
-        return {"cols": self.columns, "rows": self.get_rows()}
+        rows = self.get_rows()
+        if self._styles is None:
+            # do not apply anything
+            styles = {}
+        elif not self._styles and rows:
+            # make the last elements bold
+            styles = {len(rows) - 1: "rich"}
+        else:
+            styles = self._styles
+        return {"cols": self.columns,
+                "rows": rows,
+                "styles": styles}
 
 
 class MainStatsTable(Table):
@@ -354,35 +386,171 @@ class MainStatsTable(Table):
     columns = ["Action", "Min (sec)", "Median (sec)", "90%ile (sec)",
                "95%ile (sec)", "Max (sec)", "Avg (sec)", "Success", "Count"]
 
+    _DEPTH_OF_PROCESSING = 2
+
     def __init__(self, *args, **kwargs):
         super(MainStatsTable, self).__init__(*args, **kwargs)
-        iters_num = self._workload_info["iterations_count"]
-        for name in (list(self._workload_info["atomic"].keys()) + ["total"]):
-            self._data[name] = [
+        self.iters_num = self._workload["total_iteration_count"]
+
+    def _initialize_atomic(self, name, root, real_name=None, count=1):
+        real_name = real_name or name
+        root[name] = {
+            # streaming algorithms
+            "sa": [
                 [streaming.MinComputation(), None],
-                [streaming.PercentileComputation(0.5, iters_num), None],
-                [streaming.PercentileComputation(0.9, iters_num), None],
-                [streaming.PercentileComputation(0.95, iters_num), None],
+                [streaming.PercentileComputation(0.5, self.iters_num), None],
+                [streaming.PercentileComputation(0.9, self.iters_num), None],
+                [streaming.PercentileComputation(0.95, self.iters_num), None],
                 [streaming.MaxComputation(), None],
                 [streaming.MeanComputation(), None],
                 [streaming.MeanComputation(),
                  lambda st, has_result: ("%.1f%%" % (st.result() * 100)
                                          if has_result else "n/a")],
                 [streaming.IncrementComputation(),
-                 lambda st, has_result: st.result()]]
+                 lambda st, has_result: st.result()]],
+            "children": collections.OrderedDict(),
+            "real_name": real_name,
+            "count_per_iteration": count
+        }
 
-    def _map_iteration_values(self, iteration):
-        return dict(iteration["atomic_actions"], total=iteration["duration"])
+    def _add_data(self, raw_data, root=None):
+        """Add iteration data."""
+        p_data = self._data if root is None else root
+        for name, data in raw_data.items():
+            original_name = name
+            if data["count"] > 1:
+                name += (" (x%s)" % data["count"])
+            if name not in p_data:
+                self._initialize_atomic(name,
+                                        root=p_data,
+                                        real_name=original_name,
+                                        count=data["count"])
+
+            stats = p_data[name]["sa"]
+            # count
+            stats[-1][0].add()
+            # success
+            stats[-2][0].add(0 if data.get("failed", False) else 1)
+            for idx in range(6):
+                stats[idx][0].add(data["duration"])
+
+            if data["children"]:
+                self._add_data(data["children"], root=p_data[name]["children"])
 
     def add_iteration(self, iteration):
-        for name, value in self._map_iteration_values(iteration).items():
-            self._data[name][-1][0].add()
-            if iteration["error"]:
-                self._data[name][-2][0].add(0)
-            else:
-                self._data[name][-2][0].add(1)
-                for idx, dummy in enumerate(self._data[name][:-2]):
-                    self._data[name][idx][0].add(value)
+        """Add data of a single iteration."""
+        data = atomic.merge_atomic_actions(iteration["atomic_actions"])
+        # NOTE(andreykurilin): the easiest way to identify the last
+        #   atomic is to find the last added key to the OrderedDict. The
+        #   most perfect way is to use reversed, since class OrderedDict
+        #   uses a doubly linked list for the dictionary items and
+        #   implements __reversed__(), what is why such implementation
+        #   gives you O(1) access to the desired element.
+        if data:
+            the_last = data[next(reversed(data))]
+            if iteration["error"] and not the_last.get("failed", False):
+                # un-wrapped action failed
+                data["<no-name-action>"] = {"duration": 0, "count": 1,
+                                            "failed": True, "children": {}}
+        total_duration = iteration["duration"] + iteration["idle_duration"]
+        data["total"] = {"duration": total_duration,
+                         "count": 1,
+                         "failed": bool(iteration["error"]),
+                         "children": collections.OrderedDict(
+                             [("duration", {
+                                 "duration": iteration["duration"],
+                                 "count": 1,
+                                 "failed": bool(iteration["error"]),
+                                 "children": []}),
+                              ("idle_duration", {
+                                  "duration": iteration["idle_duration"],
+                                  "count": 1,
+                                  "failed": bool(iteration["error"]),
+                                  "children": []})
+                              ])}
+
+        self._add_data(data)
+
+    def _process_result(self, name, values, depth=0):
+        row = self._process_row(name, values["sa"])
+        children = []
+
+        for c_name, c_values in values["children"].items():
+            children.append(self._process_result(c_name, c_values))
+        return {"data": {"iteration_count": row[8],
+                         "min": row[1],
+                         "median": row[2],
+                         "90%ile": row[3],
+                         "95%ile": row[4],
+                         "max": row[5],
+                         "avg": row[6],
+                         "success": row[7]},
+                "count_per_iteration": values["count_per_iteration"],
+                "name": values["real_name"],
+                "display_name": name,
+                "children": children}
+
+    def _get_results(self):
+        if self._data:
+            # NOTE(andreykurilin): In case when the specific atomic action was
+            #   not executed in the first iteration, it will be added to
+            #   self._data after the "total" raw. It is a wrong order, so
+            #   let's ensure that the "total" is always at the end
+            self._data["total"] = self._data.pop("total")
+        else:
+            # NOTE(andreykurilin): The workload doesn't have any iteration, so
+            #   the method `add_iteration` had not been called and 'total' item
+            #   had not be initialized. Let's do it here, since 'total' item
+            #   should always present in the rows.
+            self._initialize_atomic("total", root=self._data)
+
+        results = []
+        for name, values in self._data.items():
+            results.append(self._process_result(name, values))
+        return results
+
+    def get_rows(self):
+        rows = []
+
+        def _process_elem(elem, depth=0):
+            name = elem["display_name"]
+            if depth > 0:
+                name = (" %s> %s" % ("-" * depth, name))
+            rows.append([name,
+                         elem["data"]["min"],
+                         elem["data"]["median"],
+                         elem["data"]["90%ile"],
+                         elem["data"]["95%ile"],
+                         elem["data"]["max"],
+                         elem["data"]["avg"],
+                         elem["data"]["success"],
+                         elem["data"]["iteration_count"]])
+            for child in elem["children"]:
+                _process_elem(child, depth=(depth + 1))
+
+        for elem in self._get_results():
+            _process_elem(elem)
+
+        return rows
+
+    def to_dict(self):
+        res = self._get_results()
+        return {"total": res[-1], "atomics": res[:-1]}
+
+    def render(self):
+        rendered_data = super(MainStatsTable, self).render()
+        rows_len = len(rendered_data["rows"])
+        if rows_len > 1:
+            styles = {rows_len - 3: "rich",
+                      rows_len - 2: "oblique",
+                      rows_len - 1: "oblique"}
+            for i, row in enumerate(rendered_data["rows"]):
+                if i == rows_len - 3:
+                    break
+                if row[0].startswith(" -"):
+                    styles[i] = "oblique"
+            rendered_data["styles"] = styles
+        return rendered_data
 
 
 class OutputChart(Chart):
@@ -396,9 +564,6 @@ class OutputChart(Chart):
         self.label = label
         self.axis_label = axis_label
 
-    def _map_iteration_values(self, iteration):
-        return iteration
-
     def render(self):
         return {"title": self.title,
                 "description": self.description,
@@ -556,7 +721,7 @@ class OutputStatsTable(OutputTable):
     def add_iteration(self, iteration):
         for name, value in self._map_iteration_values(iteration):
             if name not in self._data:
-                iters_num = self._workload_info["iterations_count"]
+                iters_num = self._workload["total_iteration_count"]
                 self._data[name] = [
                     [streaming.MinComputation(), None],
                     [streaming.PercentileComputation(0.5, iters_num), None],
@@ -593,6 +758,69 @@ class OutputTextArea(OutputChart):
     widget = "TextArea"
 
 
+@plugin.configure(name="EmbeddedChart")
+class OutputEmbeddedChart(OutputChart):
+    """Chart for embedding custom html as a complete chart.
+
+    Example of usage:
+
+    .. code-block:: python
+
+        self.add_output(
+            complete={
+                "title": "Embedding link to example.com",
+                "chart_plugin": "EmbeddedChart",
+                "data": "<a href='example.com'>"
+                        "To see external logs follow this link"
+                        "</a>"
+            }
+        )
+    """
+
+    widget = "EmbedChart"
+
+    @classmethod
+    def render_complete_data(cls, pdata):
+        return {
+            "title": pdata["title"],
+            "widget": cls.widget,
+            "data": {
+                "source": None,
+                # NOTE(chenxu): ensure that '</script>' of embedded_data will
+                #   not be handled incorrectly by JavaScript.
+                "embedded": pdata["data"].replace("/script>", "\\/script>"),
+            }
+        }
+
+
+@plugin.configure(name="EmbeddedExternalChart")
+class OutputEmbeddedExternalChart(OutputChart):
+    """Chart for embedding external html page as a complete chart.
+
+    Example of usage:
+
+    .. code-block:: python
+
+        self.add_output(
+            complete={
+                "title": "Embedding external html page",
+                "chart_plugin": "EmbeddedExternalChart",
+                "data": "https://example.com"
+            }
+        )
+    """
+
+    widget = "EmbedChart"
+
+    @classmethod
+    def render_complete_data(cls, pdata):
+        return {
+            "title": pdata["title"],
+            "widget": cls.widget,
+            "data": {"embedded": None, "source": pdata["data"]}
+        }
+
+
 _OUTPUT_SCHEMA = {
     "key_types": {
         "title": six.string_types,
@@ -612,7 +840,7 @@ def validate_output(output_type, output):
         return ("unexpected output type: '%s', "
                 "should be in ('additive', 'complete')" % output_type)
 
-    if type(output) != dict:
+    if not isinstance(output, dict):
         return ("%(name)s output item has wrong type '%(type)s', "
                 "must be 'dict'" % {"name": output_type,
                                     "type": type(output).__name__})
