@@ -102,7 +102,9 @@ def _process_workload(workload, workload_cfg, pos):
     for idx, itr in enumerate(workload["data"], 1):
         if itr["error"]:
             typ, msg, trace = itr["error"]
-            errors.append({"iteration": idx,
+            timestamp = dt.datetime.fromtimestamp(
+                itr["timestamp"]).isoformat(sep="\n")
+            errors.append({"iteration": idx, "timestamp": timestamp,
                            "type": typ, "message": msg, "traceback": trace})
 
         for i, additive in enumerate(itr["output"]["additive"]):
@@ -121,10 +123,9 @@ def _process_workload(workload, workload_cfg, pos):
 
         complete_charts = []
         for complete in itr["output"]["complete"]:
-            complete_chart = dict(complete)
-            chart_cls = plugin.Plugin.get(complete_chart.pop("chart_plugin"))
-            complete_chart["widget"] = chart_cls.widget
-            complete_charts.append(complete_chart)
+            chart_cls = plugin.Plugin.get(complete["chart_plugin"])
+            complete["widget"] = chart_cls.widget
+            complete_charts.append(chart_cls.render_complete_data(complete))
         complete_output.append(complete_charts)
 
         for chart in (main_area, main_hist, main_stat, load_profile,
@@ -212,7 +213,7 @@ def _make_source(tasks):
                 workload_cfg = collections.OrderedDict()
                 workload_cfg["scenario"] = {workload["name"]: workload["args"]}
                 workload_cfg["description"] = workload["description"]
-                workload_cfg["contexts"] = workload["context"]
+                workload_cfg["contexts"] = workload["contexts"]
                 workload_cfg["runner"] = {
                     workload["runner_type"]: workload["runner"]}
                 workload_cfg["hooks"] = [h["config"]
@@ -242,7 +243,7 @@ def plot(tasks_results, include_libs=False):
                            include_libs=include_libs)
 
 
-def trends(tasks):
+def trends(tasks, include_libs=False):
     trends = Trends()
     for task in tasks:
         for workload in itertools.chain(
@@ -250,7 +251,8 @@ def trends(tasks):
             trends.add_result(task["uuid"], workload)
     template = ui_utils.get_template("task/trends.html")
     return template.render(version=version.version_string(),
-                           data=json.dumps(trends.get_data()))
+                           data=json.dumps(trends.get_data()),
+                           include_libs=include_libs)
 
 
 class Trends(object):
@@ -310,7 +312,12 @@ class Trends(object):
         self._data[key]["sla_failures"] += not workload["pass_sla"]
 
         duration_stats = workload["statistics"]["durations"]
-        ts = int(workload["start_time"] * 1000)
+        if not workload["start_time"]:
+            # NOTE(andreykurilin): The workload didn't start. Probably,
+            #   one of contexts failed.
+            ts = None
+        else:
+            ts = int(workload["start_time"] * 1000)
 
         for action in itertools.chain(duration_stats["atomics"],
                                       [duration_stats["total"]]):
