@@ -24,27 +24,27 @@ class PathOrUrlTestCase(test.TestCase):
 
     @mock.patch("os.path.isfile")
     @mock.patch("requests.head")
-    def test_transform_file(self, mock_requests_head, mock_isfile):
+    def test_preprocess_file(self, mock_requests_head, mock_isfile):
         mock_isfile.return_value = True
-        path = types.PathOrUrl.transform(None, "fake_path")
+        path = types.PathOrUrl({}, {}).pre_process("fake_path", {})
         self.assertEqual("fake_path", path)
 
     @mock.patch("os.path.isfile")
     @mock.patch("requests.head")
-    def test_transform_bogus(self, mock_requests_head, mock_isfile):
+    def test_preprocess_bogus(self, mock_requests_head, mock_isfile):
         mock_isfile.return_value = False
         mock_requests_head.return_value = mock.Mock(status_code=500)
         self.assertRaises(exceptions.InvalidScenarioArgument,
-                          types.PathOrUrl.transform,
-                          None, "fake_path")
-        mock_requests_head.assert_called_once_with("fake_path")
+                          types.PathOrUrl({}, {}).pre_process,
+                          "fake_path", {})
+        mock_requests_head.assert_called_once_with("fake_path", verify=False)
 
     @mock.patch("os.path.isfile")
     @mock.patch("requests.head")
-    def test_transform_url(self, mock_requests_head, mock_isfile):
+    def test_preprocess_url(self, mock_requests_head, mock_isfile):
         mock_isfile.return_value = False
         mock_requests_head.return_value = mock.Mock(status_code=200)
-        path = types.PathOrUrl.transform(None, "fake_url")
+        path = types.PathOrUrl({}, {}).pre_process("fake_url", {})
         self.assertEqual("fake_url", path)
 
 
@@ -53,19 +53,18 @@ class FileTypeTestCase(test.TestCase):
     @mock.patch("six.moves.builtins.open",
                 side_effect=mock.mock_open(read_data="file_context"),
                 create=True)
-    def test_transform_by_path(self, mock_open):
-        resource_config = "file.yaml"
-        file_context = types.FileType.transform(
-            clients=None, resource_config=resource_config)
+    def test_preprocess_by_path(self, mock_open):
+        resource_spec = "file.yaml"
+        file_context = types.FileType({}, {}).pre_process(resource_spec, {})
         self.assertEqual("file_context", file_context)
 
     @mock.patch("six.moves.builtins.open", side_effect=IOError, create=True)
-    def test_transform_by_path_no_match(self, mock_open):
-        resource_config = "nonexistant.yaml"
+    def test_preprocess_by_path_no_match(self, mock_open):
+        resource_spec = "nonexistant.yaml"
         self.assertRaises(IOError,
-                          types.FileType.transform,
-                          clients=None,
-                          resource_config=resource_config)
+                          types.FileType({}, {}).pre_process,
+                          resource_spec=resource_spec,
+                          config={})
 
 
 class FileTypeDictTestCase(test.TestCase):
@@ -73,17 +72,15 @@ class FileTypeDictTestCase(test.TestCase):
     @mock.patch("six.moves.builtins.open",
                 side_effect=mock.mock_open(read_data="file_context"),
                 create=True)
-    def test_transform_by_path(self, mock_open):
-        resource_config = ["file.yaml"]
-        file_context = types.FileTypeDict.transform(
-            clients=None,
-            resource_config=resource_config)
+    def test_preprocess_by_path(self, mock_open):
+        resource_spec = ["file.yaml"]
+        file_context = types.FileTypeDict({}, {}).pre_process(resource_spec,
+                                                              {})
         self.assertEqual({"file.yaml": "file_context"}, file_context)
 
     @mock.patch("six.moves.builtins.open", side_effect=IOError, create=True)
-    def test_transform_by_path_no_match(self, mock_open):
-        resource_config = ["nonexistant.yaml"]
+    def test_preprocess_by_path_no_match(self, mock_open):
+        resource_spec = ["nonexistant.yaml"]
         self.assertRaises(IOError,
-                          types.FileTypeDict.transform,
-                          clients=None,
-                          resource_config=resource_config)
+                          types.FileTypeDict({}, {}).pre_process,
+                          resource_spec=resource_spec, config={})
